#include "compiler.h"

#include "Modules/LLP.h"

#include "CompileNotValidatedImpl.h"

#include "Components/ModuleValidation/ModuleValidation.h"
#include "Components/ModuleComponent/ClassDefinitionComponent.h"

std::vector<AttributeDefine*>* CompileNotValidatedClassDefine::performClassAttributes() {
	std::u32string name;
	std::vector<AttributeDefine*>* adefs = new std::vector<AttributeDefine*>;
	std::optional<Token> token;
	ExprComponent* pt = nullptr;

	while (1) {		
		token = tokenizer->copyOrEnd();
		if (token.has_value() && token->isBracket(U"}")) {
			break;
		}

		auto adef = new AttributeDefine;
		adefs->push_back(adef);

		bool is_export = false;
		if (token.has_value() && token->isKeyword(U"导出")) {
			is_export = true;
			token = tokenizer->dropAndCopy();
		}

		bool is_static = false;
		if (token.has_value() && token->isKeyword(U"静态存储")) {
			is_static = true;
			token = tokenizer->dropAndCopy();
		}

		bool is_edge = false;
		if (token.has_value() && token->isKeyword(U"边界继承")) {
			is_edge = true;
			token = tokenizer->dropAndCopy();
		}

		pt = performFormula();
		if (pt == nullptr) {
			goto error;
		}
		if (!queryLLP()->Runtime()->valid->checkSpecifiedClassNotionSyntax(compileProcess->filePath, pt)) {
			goto error;
		}

		token = tokenizer->copyOrEnd();
		if (!token.has_value() || !token->isIdentifier()) {
			pushError(u8"不是一个合法的标识符");
			goto error;
		}
		name = token->value;
		token = tokenizer->dropAndCopy();

		if (token.has_value() && token->isBracket(U"(")) {
			auto fdef = performFunctionDefinition(is_export, is_static, is_edge, pt, name);
			if (fdef == nullptr) {
				goto error;
			}
			adef->is_function = true;
			adef->func = fdef;
		}
		else {
			if (is_edge) {
				pushError(u8"属性值不允许边界继承");
				goto error;
			}
			token = tokenizer->copyOrEnd();
			if (!token.has_value() || !token->testRandom(U";")) {
				pushError(u8"非预期的输入序列终止");
				goto error;
			}
			tokenizer->dropOrEnd();
			adef->cls = pt;
			adef->is_export = is_export;
			adef->is_static = is_static;
			adef->name = name;
		}
	}
	return adefs;
error:
	if (pt != nullptr) {
		delete pt;
	}
	if (adefs != nullptr) {
		for (auto s : *adefs) {
			delete s;
		}
	}
	return nullptr;
}

ClassDefinitionComponent* CompileNotValidatedClassDefine::performClassDefineSyntax(){

	auto cdef = new ClassDefinitionComponent;

	auto token = tokenizer->copyOrEnd();

	if (token->isKeyword(U"导出")) {
		cdef->is_export = true;
		token = tokenizer->dropAndCopy();
	}
	if (token->isKeyword(U"可以之类")) {
		token = tokenizer->dropAndCopy();
	}
	else if (token->isKeyword(U"守拙之类")) {
		cdef->is_second = true;
		token = tokenizer->dropAndCopy();
	}
	else {
		pushError(u8"无法处理的输入，不是一个类定义");
		goto error;
	}

	if (!token.has_value() || !token->isIdentifier()) {
		pushError(u8"未找到类名称");
		goto error;
	}
	cdef->token = *token;
	cdef->name = token->value;
	token = tokenizer->dropAndCopy();

	if (token.has_value() && token->isBracket(U"[")) {
		cdef->tp = performBracket(U"[", U"]");
		if (cdef->tp == nullptr) {
			goto error;
		}
		if (!queryLLP()->Runtime()->valid->checkTypeCaptureSyntax(compileProcess->filePath, cdef->tp)) {
			goto error;
		}
		cdef->is_template = true;
	}

	token = tokenizer->copyOrEnd();
	if (token->isKeyword(U"表现")) {
		tokenizer->dropOrEnd();
		cdef->ep = performCommaChunk();
		if (cdef->ep == nullptr) {
			goto error;
		}
		if (!queryLLP()->Runtime()->valid->checkExpressionClassSyntax(compileProcess->filePath, *cdef->ep)) {
			goto error;
		}
	}

	token = tokenizer->copyOrEnd();
	if (token->testRandom(U";")) {
		tokenizer->dropOrEnd();
		return cdef;
	}

	if (!token->isBracket(U"{")) {
		pushError(u8"未找到左括号");
		goto error;
	}

	tokenizer->dropOrEnd();

	if ((cdef->attrs = performClassAttributes()) == nullptr) {
		goto error;
	}

	token = tokenizer->copyOrEnd();
	if (!token->isBracket(U"}")) {
		pushError(u8"未找到右括号");
		goto error;
	}

	tokenizer->dropOrEnd();

	return cdef;

error:
	if (cdef != nullptr) {
		delete cdef;
	}
	return nullptr;
};