#include "compiler.h"

#include "CompileNotValidatedImpl.h"

#include "Components/ModuleValidation/ModuleValidation.h"
#include "Components/ModuleComponent/FunctionDefinitionComponent.h"

#include "Modules/LLP.h"

FunctionDefinitionComponent* CompileNotValidatedFunctionDefine::performFunctionDefinition(bool is_export, bool is_static, bool is_edge, ExprComponent* pt, std::u32string name) {

	FunctionDefinitionComponent* func_def = nullptr;
	FunctionDefinitionML* func_ml = nullptr;

	auto token = tokenizer->copyOrEnd();

	func_def = new FunctionDefinitionComponent;
	func_ml = func_def->ml = new FunctionDefinitionML;

	func_ml->name = name;

	if ((func_ml->sp = performBracket(U"(", U")")) == nullptr) {
		goto error;
	}
	if (!queryLLP()->Runtime()->valid->checkFunctionParamsListSyntax(compileProcess->filePath, func_ml->sp)) {
		goto error;
	}

	func_ml->is_export = is_export;
	func_ml->is_static = is_static;
	func_ml->is_edge = is_edge;
	func_ml->rt = pt;

	token = tokenizer->copyOrEnd();

	if (!token.has_value()) {
		pushError(u8"非预期的输入序列终止");
		goto error;
	}
	if (token->testRandom(U"&")) {
		func_ml->mutable_mark = true;
		token = tokenizer->dropAndCopy();
	}
	if (token->isKeyword(U"演绎")) {
		func_ml->perform_mark = true;
		token = tokenizer->dropAndCopy();
	}

	if (token.has_value() && token->testRandom(U";")) {
		tokenizer->dropOrEnd();
		return func_def;
	}

	func_ml->S = performBlock();

	if (func_ml->S == nullptr) {
		goto error;
	}

	return func_def;

error:
	if (func_def != nullptr) {
		delete func_def;
	}
	if (func_ml != nullptr) {
		delete func_ml;
	}
	return nullptr;
}

FunctionDefinitionComponent* CompileNotValidatedFunctionDefine::performFunctionDefinition() {
	FunctionDefinitionComponent* fdef;
	std::u32string name;
	auto token = tokenizer->copyOrEnd();

	bool is_export = false;
	if (token.has_value() && token->isKeyword(U"导出")) {
		is_export = true;
		token = tokenizer->dropAndCopy();
	}

	bool is_static = false;
	if (token.has_value() && token->isKeyword(U"静态存储")) {
		is_static = true;
		token = tokenizer->dropAndCopy();
	}

	bool is_edge = false;
	if (token.has_value() && token->isKeyword(U"边界继承")) {
		is_edge = true;
		token = tokenizer->dropAndCopy();
	}

	auto pt = performFormula();
	if (pt == nullptr) {
		goto error;
	}
	if (!queryLLP()->Runtime()->valid->checkSpecifiedClassNotionSyntax(compileProcess->filePath, pt)) {
		goto error;
	}

	token = tokenizer->copyOrEnd();
	if (!token.has_value() || !token->isIdentifier()) {
		pushError(u8"不是一个合法的标识符");
		goto error;
	}
	name = token->value;
	token = tokenizer->dropAndCopy();

	fdef = performFunctionDefinition(is_export, is_static, is_edge, pt, name);
	if (fdef == nullptr) {
		goto error;
	}
	return fdef;

error:
	if (pt != nullptr) {
		delete pt;
	}
	return nullptr;
};