#include "CompileNotValidatedImpl.h"

#include "unicode.h"

#include "Components/ModuleComponent/StatementComponent.h"

bool matchKeywordAndDrop(Tokenizer& tokenizer, const std::u32string& s) {
	auto token = tokenizer.copyOrEnd();
	if (token.has_value() && token->isKeyword(s)) {
		tokenizer.dropOrEnd();
		return true;
	}
	else {
		if (!token.has_value()) {
			tokenizer.pushError(-1, -1, u8"未找到关键字\"" + To_UTF8(s) + u8"\"");
		}
		else {
			tokenizer.pushError(token->line_pos, token->column_pos, u8"未找到关键字\"" + To_UTF8(s) + u8"\"");
		}
		return false;
	}
}

bool matchBracketAndDrop(Tokenizer& tokenizer, const std::u32string& s) {
	auto token = tokenizer.copyOrEnd();
	if (token.has_value() && token->isBracket(s)) {
		tokenizer.dropOrEnd();
		return true;
	}
	else {
		tokenizer.pushError(token->line_pos, token->column_pos, u8"未找到括号\"" + To_UTF8(s) + u8"\"");
		return false;
	}
}

bool matchRandomAndDrop(Tokenizer& tokenizer, const std::u32string& s) {
	auto token = tokenizer.copyOrEnd();
	if (token.has_value() && token->testRandom(s)) {
		tokenizer.dropOrEnd();
		return true;
	}
	else {
		tokenizer.pushError(token->line_pos, token->column_pos, u8"未找到\"" + To_UTF8(s) + u8"\"");
		return false;
	}
}

StatementComponent* CompileNotValidatedStatement::performExprStatment() {
	auto stmt = new StatementComponent(StatementComponentKind::expr_statement);
	if ((stmt->SC[0] = performExpr()) == nullptr) {
		delete stmt;
		return nullptr;
	}
	if (!matchRandomAndDrop(*tokenizer, U";")) {
		delete stmt;
		return nullptr;
	}
	return stmt;
}

StatementComponent* CompileNotValidatedStatement::performIf() {
	StatementComponent* stmt = nullptr;
	std::optional<Token> token;
	if (!matchKeywordAndDrop(*tokenizer, U"举")) goto error;
	stmt = new StatementComponent(StatementComponentKind::if_statement);
	if ((stmt->SC[0] = performBracket(U"(", U")")) == nullptr) goto error;
	if ((stmt->S[0] = performBlock()) == nullptr) goto error;
	token = tokenizer->copyOrEnd();
	if (token.has_value() && token->isKeyword(U"伪")) {
		tokenizer->dropOrEnd();
		if ((stmt->S[1] = performBlock()) == nullptr) goto error;
	}
	return stmt;
error:
	if (stmt != nullptr) {
		delete stmt;
	}
	return nullptr;
}

StatementComponent* CompileNotValidatedStatement::performFor() {
	StatementComponent* stmt = nullptr;
	bool iter_for = false;
	std::optional<Token> token;
	if (!matchKeywordAndDrop(*tokenizer, U"推")) goto error;
	if (!matchBracketAndDrop(*tokenizer, U"(")) goto error;
	stmt = new StatementComponent(StatementComponentKind::for_statement);
	for (int i = 0; i < 3; ++i) {
		if (i > 0) {
			if (iter_for && (i >= 2)) {
				break;
			}
			auto token = tokenizer->copyOrEnd();
			if ((i == 1) && (token->testRandom(U":"))) {
				iter_for = true;
				stmt->kind = StatementComponentKind::for_iter_statement;
				tokenizer->dropOrEnd();
			}
			else if (!matchRandomAndDrop(*tokenizer, U";")){
				pushError(u8"未找到分号");
				goto error;
			}
		}
		if ((stmt->SC[i] = performExpr()) == nullptr) {
			goto error;
		}
	}
	if (!matchBracketAndDrop(*tokenizer, U")")) goto error;
	if ((stmt->S[0] = performBlock()) == nullptr) {
		goto error;
	}
	return stmt;
error:
	if (stmt != nullptr) {
		delete stmt;
	}
	return nullptr;
}

StatementComponent* CompileNotValidatedStatement::performWhile() {
	StatementComponent* stmt = nullptr;
	if (!matchKeywordAndDrop(*tokenizer, U"伺")) goto error;
	stmt = new StatementComponent(StatementComponentKind::while_statement);
	if((stmt->S[0] = performBlock()) == nullptr) goto error;
	return stmt;
error:
	if (stmt != nullptr) {
		delete stmt;
	}
	return nullptr;
}

StatementComponent* CompileNotValidatedStatement::performJump() {
	StatementComponent* stmt = nullptr;
	auto token = tokenizer->copyOrEnd();
	if (!token.has_value()) {
		pushError(u8"非预期的输入结束");
		goto error;
	}
	if (token->isKeyword(U"避")) {
		stmt = new StatementComponent(StatementComponentKind::continue_statement);
		tokenizer->dropOrEnd();
	}
	if (token->isKeyword(U"退")) {
		stmt = new StatementComponent(StatementComponentKind::break_statement);
		tokenizer->dropOrEnd();
	}
	if (token->isKeyword(U"返回")) {
		stmt = new StatementComponent(StatementComponentKind::return_statement);
		token = tokenizer->dropAndCopy();
		if (token.has_value() && token->testRandom(U";")) {

		}
		else {
			if ((stmt->SC[0] = performExpr()) == nullptr) goto error;
		}
	}
	if (!matchRandomAndDrop(*tokenizer, U";")) {
		goto error;
	}
	return stmt;
error:
	if (stmt != nullptr) {
		delete stmt;
	}
	return nullptr;
}

StatementComponent* CompileNotValidatedStatement::performBlock() {
	StatementComponent* stmt = nullptr;
	std::optional<Token> token;
	if (!matchBracketAndDrop(*tokenizer, U"{")) goto error;
	stmt = new StatementComponent(StatementComponentKind::block_statement);
	stmt->sp = new std::vector<StatementComponent*>;
	while (1) {
		token = tokenizer->copyOrEnd();
		if (token.has_value() && token->isBracket(U"}")) {
			break;
		}
		auto stmt1 = performStatement();
		if (stmt1 == nullptr) goto error;
		stmt->sp->push_back(stmt1);
	}
	tokenizer->dropOrEnd();
	return stmt;
error:
	if (stmt != nullptr) {
		delete stmt;
	}
	return nullptr;
}

StatementComponent* CompileNotValidatedStatement::performStatement() {
	auto token = tokenizer->copyOrEnd();
	/* 空语句 */
	if (token->testRandom(U";")) {
		tokenizer->dropOrEnd();
		return new StatementComponent(StatementComponentKind::empty_statement);
	}
	/* 条件语句 */
	if (token->isKeyword(U"举")) {
		return performIf();
	}
	/* for语句 */
	if (token->isKeyword(U"推")) {
		return performFor();
	}
	/* while语句 */
	if (token->isKeyword(U"伺")) {
		return performWhile();
	}
	/* jump语句 */
	if (token->isKeyword(U"避")
		|| token->isKeyword(U"退")
		|| token->isKeyword(U"返回")) {
		return performJump();
	}
	/* 块语句 */
	if (token->isBracket(U"{")) {
		return performBlock();
	}
	/* 值语句 */
	return performExprStatment();
}