#include "compiler.h"

#include "Modules/LLP.h"

#include "CppBuilder.h"

#include "unicode.h"

#include "Components/ModuleComponent/ExprComponent.h"
#include "Components/ModuleComponent/ClassDefinitionComponent.h"
#include "Components/ModuleComponent/ModuleComponent.h"
#include "Components/ModuleCache/ModuleCache.h"

#include <filesystem>

std::u8string HeadFileName(ModuleComponent* t) {
	std::u8string buf;
	auto pt = t->package_name;
	for (int i = 0; i < pt.size(); ++i) {
		if (i > 0) {
			buf += u8"/";
		}
		buf += To_UTF8(pt[i]);
	}
	{
		std::filesystem::path fp = t->file_path;
		buf += u8"\\" + fp.stem().u8string();
	}
	return buf;
}


void Build_HeadFile_Include(std::u8string& buf, ModuleComponent* t) {
	auto & mcache = *(queryLLP()->Runtime()->mcache);
	for (auto r : t->import_module) {
		auto mod = mcache.queryModuleCache(r);
		if (mod) {
			buf += u8"#include \"" + HeadFileName(mod) + u8".h\"\n\n";
		}
	}
}

std::u8string template_arguments(ClassDefinitionComponent* c) {
	std::u8string tmpl;
	auto sp = *(c->tp->sp);
	for (int i = 0; i < sp.size(); ++i) {
		if (i > 0) {
			tmpl += u8", ";
		}
		tmpl += u8"typename " + To_UTF8(sp[i]->S[0]->identifier);
	}
	return tmpl;
}

std::u8string expr_string_1(ExprComponent* expr) {
	std::u8string buf;
	if (expr->kind == ExprComponentKind::unary) {
		buf += To_UTF8(expr->op);
		buf += expr_string_1(expr->S[0]);
	}
	else if (expr->kind == ExprComponentKind::binary) {
		if (expr->op == U"postfix.[]") {
			buf += expr_string_1(expr->S[0]);
			buf += expr_string_1(expr->S[1]);
		}
		else if (expr->op == U"postfix.()") {
			buf += expr_string_1(expr->S[0]);
			buf += expr_string_1(expr->S[1]);
		}
		else {
			buf += u8"(" + expr_string_1(expr->S[0]);
			buf += To_UTF8(expr->op);
			buf += expr_string_1(expr->S[1]) + u8")";
		}
	}
	else if (expr->kind == ExprComponentKind::quantity) {
		buf += To_UTF8(expr->quantity);
	}
	else if (expr->kind == ExprComponentKind::identifier) {
		buf += To_UTF8(expr->identifier);
	}
	else if (expr->kind == ExprComponentKind::bracket) {
		buf += (char8_t) expr->op[0];
		for (int i = 0; i < expr->sp->size(); ++i) {
			if (i > 0) { buf += u8", "; }
			buf += expr_string_1((*expr->sp)[i]);
		}
		buf += (char8_t) expr->op[1];
	}
	else if (expr->kind == ExprComponentKind::chunk) {
		for (int i = 0; i < expr->sp->size(); ++i) {
			if (i > 0) { buf += u8" "; }
			buf += expr_string_1((*expr->sp)[i]);
		}
	}
	else {
		buf += u8"???ERROR";
	}
	return buf;
}

bool isTypeParameter(ClassDefinitionComponent* c, std::u32string a) {
	if ((c != nullptr) && (c->is_template) && (c->tp != nullptr) && (c->tp->sp != nullptr)) {
		for (auto pv : *(c->tp->sp)) {
			if (pv->S[0]->identifier == a) {
				return true;
			}
		}
	}
	return false;
}

std::u8string ultra_type_string(ClassDefinitionComponent* c, ExprComponent* s)
{
	if (c == nullptr) {
		return u8"ERRORTYPE";
	}
	if (s == nullptr) {
		std::u8string buf = To_UTF8(c->name) + u8"_Ultra";
		if ((c->is_template) && (c->tp != nullptr) && (c->tp->sp != nullptr)) {
			buf += u8"<";
			auto & sp = *(c->tp->sp);
			for (int i = 0; i < sp.size(); ++i) {
				if (i > 0) { buf += u8", "; }
				buf += To_UTF8(sp[i]->S[0]->identifier);
			}
			buf += u8">";
		}
		return buf;
	}
	else if (s->kind == ExprComponentKind::identifier) {
		if (isTypeParameter(c, s->identifier)) {
			return To_UTF8(s->identifier);
		}
		return To_UTF8(s->identifier) + u8"_Ultra";
	}
	else if (s->kind == ExprComponentKind::binary) {
		std::u8string buf;
		if (isTypeParameter(c, s->S[0]->identifier)) {
			buf += To_UTF8(s->S[0]->identifier);
		}
		else {
			buf += To_UTF8(s->S[0]->identifier) + u8"_Ultra";
		}
		buf += u8"<";
		bool first = true;
		for (auto pv : *(s->S[1]->sp)) {
			if (first) {
				first = false;
			}
			else {
				buf += u8", ";
			}
			buf += ultra_type_string(c, pv);
		}
		buf += u8">";
		return buf;
	}
	return u8"ERRORTYPE";
}

std::u8string statement_string(int indent, StatementComponent* statement) {
	std::u8string buf;
	buf += std::u8string(indent, ' ');
	if (statement->kind == StatementComponentKind::empty_statement) {
		buf += u8";";
	}
	else if (statement->kind == StatementComponentKind::expr_statement) {
		buf += expr_string_1(statement->SC[0]);
		buf += u8";";
	}
	else if (statement->kind == StatementComponentKind::if_statement) {
		buf += u8"if";
		buf += expr_string_1(statement->SC[0]);
		buf += u8"\n";
		buf += statement_string(indent, statement->S[0]);
		if (statement->S[1] != nullptr) {
			buf += u8"\n" + std::u8string(indent, ' ') + u8"else\n";
			buf += statement_string(indent, (statement->S[1]));
		}
	}
	else if (statement->kind == StatementComponentKind::for_statement) {
		buf += u8"for";
		buf += expr_string_1(statement->SC[0]);
		buf += u8";";
		buf += expr_string_1(statement->SC[1]);
		buf += u8";";
		buf += expr_string_1(statement->SC[2]);
		buf += u8")\n";
		buf += statement_string(indent, statement->S[0]);
	}
	else if (statement->kind == StatementComponentKind::for_iter_statement) {
		buf += u8"for";
		buf += expr_string_1(statement->SC[0]);
		buf += u8" : ";
		buf += expr_string_1(statement->SC[1]);
		buf += u8")\n";
		buf += statement_string(indent, statement->S[0]);
	}
	else if (statement->kind == StatementComponentKind::while_statement) {
		buf += u8"while(1)\n";
		buf += statement_string(indent, statement->S[0]);
	}
	else if (statement->kind == StatementComponentKind::continue_statement) {
		buf += u8"continue;";
	}
	else if (statement->kind == StatementComponentKind::break_statement) {
		buf += u8"break;";
	}
	else if (statement->kind == StatementComponentKind::return_statement) {
		buf += u8"return";
		if (statement->SC[0]) {
			buf += u8" ";
			buf += expr_string_1(statement->SC[0]);
		}
		buf += u8";";
	}
	else if (statement->kind == StatementComponentKind::block_statement) {
		buf += u8"{\n";
		for (auto s : *statement->sp) {
			buf += statement_string(indent+2, s);
			buf += u8"\n";
		}
		buf += std::u8string(indent, ' ') + u8"}";
	}
	else {
		buf += u8"??ERROR";
	}
	return buf;
}

std::u8string Build_Attribute(std::u8string& buf, AttributeDefine* attr, ClassDefinitionComponent* cdef, bool build_prototype, bool build_ultra, bool is_template, bool is_source) {
	if (attr->is_function) {
		auto fdef = attr->func;
		if (!is_source) {
			buf += u8"virtual ";
		}
		if (fdef->ml->is_static) {
			buf += u8"static ";
		}
		buf += expr_string_1(fdef->ml->rt);
		buf += u8" ";
		if (is_source) {
			buf += To_UTF8(cdef->name) + u8"::";
		}
		buf += To_UTF8(fdef->ml->name);
		buf += expr_string_1(fdef->ml->sp);
		if (!fdef->ml->mutable_mark) {
			buf += u8" const";
		}
		if (!is_source) {
			if (build_prototype && build_ultra) {
				buf += u8" = 0";
			}
			else {
				buf += u8" override";
			}
		}
		if (build_prototype) {
			buf += u8";";
		}
		else {
			buf += u8"\n";
			if (!is_source) {
				buf += statement_string(4, fdef->ml->S);
			}
			else {
				buf += statement_string(0, fdef->ml->S);
			}
		}
	}
	else {
		if (attr->is_static) {
			buf += u8"static ";
		}
		buf += expr_string_1(attr->cls);
		buf += u8" ";
		if (is_source) {
			buf += To_UTF8(cdef->name) + u8"::";
		}
		buf += To_UTF8(attr->name);
		buf += u8";";
	}
	return buf;
}

void Build_Classdef_HeadFile_Forward(std::u8string& buf, ClassDefinitionComponent* c) {
	if (c->is_template) {
		buf += u8"template <";
		buf += template_arguments(c);
		buf += u8">\n";
	}
	buf += u8"class " + To_UTF8(c->name) + u8"_Ultra;\n\n";

	if (c->is_template) {
		buf += u8"template <";
		buf += template_arguments(c);
		buf += u8">\n";
	}
	buf += u8"class " + To_UTF8(c->name) + u8";\n";
}

void Build_Classdef_HeadFile(std::u8string& buf, ClassDefinitionComponent* c) {
	std::u8string tmpl;
	if (c->is_template) {
		buf += u8"template <";
		buf += template_arguments(c);
		buf += u8">\n";
	}
	buf += u8"class " + To_UTF8(c->name) + u8"_Ultra {\n\n";
	for (auto attr : (*c->attrs)) {
		if (attr->is_function) {
			buf += u8"  ";
			Build_Attribute(buf, attr, c, true, true, c->is_template, false);
			buf += u8"\n\n";
		}
	}
	buf += u8"};\n\n";

	if (c->is_template) {
		buf += u8"template <";
		buf += template_arguments(c);
		buf += u8">\n";
	}
	buf += u8"class " + To_UTF8(c->name);
	buf += u8" : ";
	{
		int first = true;
		if (c->ep != nullptr) {
			for (auto s : *c->ep) {
				if (first) {
					first = false;
				}
				else {
					buf += u8", ";
				}
				buf += u8"public " + ultra_type_string(c, s);
			}
		}
		if (first) {
			first = false;
		}
		else {
			buf += u8", ";
		}
		buf += u8"public " + ultra_type_string(c, nullptr);
	}
	buf += u8" {\n\n";
	buf += u8"public:\n\n";
	buf += u8"    using UltraType = " + ultra_type_string(c, nullptr) + u8";\n\n";
	buf += u8"public:\n\n";
	for (auto attr : (*c->attrs)) {
		buf += u8"    ";
		if (c->is_template) {
			Build_Attribute(buf, attr, c, false, false, true, false);
		}
		else {
			Build_Attribute(buf, attr, c, true, false, false, false);
		}
		buf += u8"\n\n";
	}
	buf += u8"};\n";
	return;
}

void Build_Classdef(std::u8string& buf, ClassDefinitionComponent* c) {
	if (!c->is_template) {
		buf += u8"/* Gen code for " + To_UTF8(c->name) + u8" */\n\n";
		for (auto attr : (*c->attrs)) {
			if (attr->is_function) {
				Build_Attribute(buf, attr, c, false, false, false, true);
				buf += u8"\n\n";
			}
			else if (attr->is_static) {
				Build_Attribute(buf, attr, c, false, false, false, true);
				buf += u8"\n\n";
			}
		}
	}
}

std::u8string CppBuilder::buildHeadFile(ModuleComponent* Module, bool primitiveTypes) {
	std::u8string buf;
	if (primitiveTypes) {
		buf += u8"/* Primitive Types */\n\n";
		buf += u8"typedef double 浮点数;\n\n";
		buf += u8"typedef bool 布尔量;\n\n";
		buf += u8"typedef int 整数;\n\n";
	}
	buf += u8"/* Head Files */\n\n";
	Build_HeadFile_Include(buf, Module);
	buf += u8"\n";
	buf += u8"/* Class Forward */\n\n";
	for (auto c : Module->class_def) {
		Build_Classdef_HeadFile_Forward(buf, c);
		buf += u8"\n";
	}
	buf += u8"\n";
	buf += u8"/* Class */\n\n";
	for (auto c : Module->class_def) {
		if (c->attrs != nullptr) {
			Build_Classdef_HeadFile(buf, c);
			buf += u8"\n";
		}
	}
	return buf;
}

std::u8string CppBuilder::buildCppFile(ModuleComponent* Module) {
	std::u8string buf;
	buf += u8"/* Head Files */\n\n";
	buf += u8"#include \"" + HeadFileName(Module) + u8".h\"\n";
	for (auto c : Module->class_def) {
		if (c->attrs != nullptr) {
			Build_Classdef(buf, c);
			buf += u8"\n";
		}
	}
	return buf;
}

#if 0

void Build_Cpp_Expr(std::u8string& buf, elpModule* t, elpClass* c,
	elp::classes::classdef_elp::FunctionDefinition* f,
	elp::classes::classdef_elp::elpExpr* expr,
	const std::u8string& tIndent);

void Build_Binary_Bracket(std::u8string& buf, elpModule* t, elpClass* c,
	elp::classes::classdef_elp::FunctionDefinition* f,
	elp::classes::classdef_elp::elpExpr* expr) {
	using elp::classes::classdef_elp::elpExprKind;
	if (expr->kind == elpExprKind::binary) {
		buf += u8"(";
		Build_Cpp_Expr(buf, t, c, f, expr, u8"");
		buf += u8")";
	}
	else {
		Build_Cpp_Expr(buf, t, c, f, expr, u8"");
	}
}

void Build_Cpp_Expr(std::u8string& buf, elpModule* t, elpClass* c,
	elp::classes::classdef_elp::FunctionDefinition* f,
	elp::classes::classdef_elp::elpExpr* expr,
	const std::u8string& tIndent) {
	buf += tIndent;
	using elp::classes::classdef_elp::elpExprKind;
	switch (expr->kind)
	{
	case elpExprKind::quantity: /* TODO */ break;
	case elpExprKind::identifier: buf += To_UTF8(expr->identifier); break;
	case elpExprKind::unary: {
		if (expr->identifier == U"++&") {
			Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
			buf += u8" ++";
			break;
		}
		if (expr->identifier == U"--&") {
			Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
			buf += u8" --";
			break;
		}
		buf += To_UTF8(expr->identifier);
		buf += u8"(";
		Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
		buf += u8")";
		break;
	}
	case elpExprKind::binary: {
		if ((expr->identifier == U".")
			|| (expr->identifier == U"->")) {
			Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
			buf += To_UTF8(expr->identifier);
			Build_Cpp_Expr(buf, t, c, f, expr->clause[1], u8"");
			break;
		}
		Build_Binary_Bracket(buf, t, c, f, expr->clause[0]);
		buf += To_UTF8(expr->identifier);
		Build_Binary_Bracket(buf, t, c, f, expr->clause[1]);
		break;
	}
	case elpExprKind::bracket:
	{
		buf += u8"(";
		Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
		buf += u8")";
		break;
	}
	case elpExprKind::subscript:
	{
		Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
		buf += u8"[";
		for (int i = 0; i < expr->function_params.size(); ++i) {
			if (i > 0) {
				buf += u8", ";
			}
			Build_Cpp_Expr(buf, t, c, f, expr->function_params[i], u8"");
		}
		buf += u8"]";
		break;
	}
	case elpExprKind::typename_expr:
	{
		Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
		if (expr->type_params.size() > 0) {
			buf += u8"<";
			for (int i = 0; i < expr->type_params.size(); ++i) {
				if (i > 0) {
					buf += u8", ";
				}
				buf += ResolveClassDefToNotion(expr->type_params[i], false);
			}
			buf += u8">";
		}
		break;
	}
	case elpExprKind::funccall:
		Build_Cpp_Expr(buf, t, c, f, expr->clause[0], u8"");
		buf += u8"(";
		for (int i = 0; i < expr->function_params.size(); ++i) {
			if (i > 0) {
				buf += u8", ";
			}
			Build_Cpp_Expr(buf, t, c, f, expr->function_params[i], u8"");
		}
		buf += u8")";
		break;
	default:
		break;
	}
}

void Bulid_Cpp_Statement(std::u8string& buf, elpModule* t, elpClass* c,
	elp::classes::classdef_elp::FunctionDefinition* f,
	elp::classes::classdef_elp::elpStatement* sp,
	const std::u8string& tIndent,
	bool end) {
	using elp::classes::classdef_elp::elpStatementKind;
	if (sp->kind != elpStatementKind::block_statement) {
		buf += tIndent;
	}
	switch (sp->kind)
	{
	case elpStatementKind::empty_statement: if (end) { buf += u8";\n"; } break;
	case elpStatementKind::init_statement: 
	{
		Build_Cpp_Expr(buf, t, c, f, sp->expr[0], u8"");
		buf += u8" ";
		Build_Cpp_Expr(buf, t, c, f, sp->expr[1], u8""); 
		if (end) { buf += u8";\n"; }
		break;
	}
	case elpStatementKind::assignment_statement:
	{
		Build_Cpp_Expr(buf, t, c, f, sp->expr[0], u8"");
		buf += u8"=";
		Build_Cpp_Expr(buf, t, c, f, sp->expr[1], u8"");
		if (end) { buf += u8";\n"; }
		break;
	}
	case elpStatementKind::expr_statement:
	{
		Build_Cpp_Expr(buf, t, c, f, sp->expr[0], u8"");
		if (end) { buf += u8";\n"; }
		break;
	}
	case elpStatementKind::if_statement:
	{
		buf += u8"if(";
		Build_Cpp_Expr(buf, t, c, f, sp->expr[0], u8"");
		buf += u8")\n";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[0], tIndent + u8"  ", true);
		if (sp->clause[1] != nullptr) {
			buf += tIndent + u8"else\n" ;
			Bulid_Cpp_Statement(buf, t, c, f, sp->clause[1], tIndent + u8"  ", true);
		}
		break;
	}
	case elpStatementKind::for_statement:
	{
		buf += u8"for(";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[0], u8"", false);
		buf += u8";";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[1], u8"", false);
		buf += u8";";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[2], u8"", false);
		buf += u8")\n";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[3], tIndent + u8"  ", true);
		break;
	}
	case elpStatementKind::for_iter_statement:
	{
		buf += u8"for(";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[0], u8"", false);
		buf += u8":";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[1], u8"", false);
		buf += u8")\n";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[3], tIndent + u8"  ", true);
		break;
	}
	case elpStatementKind::while_statement:
	{
		buf += u8"while(1)\n";
		Bulid_Cpp_Statement(buf, t, c, f, sp->clause[0], tIndent + u8"  ", true);
		break;
	}
	case elpStatementKind::break_statement: { buf += u8"break;\n"; break;	}
	case elpStatementKind::continue_statement: { buf += u8"continue;\n"; break; }
	case elpStatementKind::return_statement:
	{
		buf += u8"return";
		if (sp->expr[0] != nullptr) {
			buf += u8" ";
			Build_Cpp_Expr(buf, t, c, f, sp->expr[0], u8"");
		}
		buf += u8";\n";
		break;
	}
	case elpStatementKind::block_statement:
	{
		buf += tIndent.substr(2) + u8"{\n";
		for (auto s : *sp->block) {
			Bulid_Cpp_Statement(buf, t, c, f, s, tIndent, true);
		}
		buf += tIndent.substr(2) + u8"}\n";
		break;
	}
	default:
		break;
	}
}

void Build_Function_Block(std::u8string& buf, elpModule* t, elpClass* c,
	elp::classes::classdef_elp::FunctionDefinition* f,
	const std::u8string& tIndent) {
	buf += u8"{\n";
	for (auto s : f->visitExecutable()->nodes) {
		Bulid_Cpp_Statement(buf, t, c, f, s, tIndent, true);
	}
	buf += tIndent.substr(2) + u8"}\n";
}

void Build_Tmpldef_HeadFile(std::u8string& buf, elpModule* t, elpClass* c) {
	if (!c->getClassMark()->assemblied_class) {
		return;
	}
	Build_Classdef_HeadFile_Forward(buf, t, c);
	Build_Class_Prototype(buf, t, c, u8"  ");
	for (int k = 0; k < 2; ++k) {
		if (k == 0) { Build_Class_Label(buf, U"public", u8"  "); };
		if (k == 1) { Build_Class_Label(buf, U"private", u8"  "); };
		for (auto f : c->queryFunctions()) {
			auto mark = f->getMark();
			if ((k == 0) && (!mark.export_function)) { continue; }
			if ((k == 1) && (mark.export_function)) { continue; }
			if (f->visitExecutable() == nullptr) {
				continue;
			}
			Build_Function_Forward(buf, f, t, u8"  ");
			buf += u8" ";
			Build_Function_Block(buf, t, c, f, u8"        ");
			buf += u8"\n";
		}
	}
	buf += u8"  }\n";
}

std::u8string CppBuilder::buildHeadFile(elpModule* t) {
	std::u8string buf;

	buf += u8"#ifndef " + ResolveDotExprToMACRO(t->getDotExpr()) + u8"_H\n";
	buf += u8"#define " + ResolveDotExprToMACRO(t->getDotExpr()) + u8"_H\n";
	buf += u8"#define _HeadFileScope\n\n";
	
	Build_HeadFile_Include(buf, t);
	buf += u8"\nnamespace " + ResolveDotExprToNS(t->getDotExpr(), true) + u8" {\n\n";
	Build_UsingNS_Code(buf, t);
	buf += u8"\n";
	buf += u8"  /* 导出类 */\n\n";
	for (auto c : *t->queryAllClasses()) {
		if (c->getClassMark()->export_class) {
			Build_Classdef_HeadFile_Forward(buf, t, c);
			buf += u8";\n\n";
		}
	}
	buf += u8"  /* 非导出类 */\n\n";
	for (auto c : *t->queryAllClasses()) {
		if (!c->getClassMark()->export_class) {
			Build_Classdef_HeadFile_Forward(buf, t, c);
			buf += u8";\n\n";
		}
	}
	buf += u8"  /* 导出类的结构 */\n\n";
	for (auto c : *t->queryAllClasses()) {
		if (c->getClassMark()->export_class) {
			if (c->getClassMark()->subscript_class) {
				continue;
			}
			Build_Classdef_HeadFile(buf, t, c);
			buf += u8"\n";
		}
	}
	buf += u8"  /* 类模板 */\n\n";
	for (auto c : *t->queryAllClasses()) {
		if (c->getClassMark()->export_class
			&& c->getClassMark()->subscript_class
			&& c->isDefinitiveClass()){
			Build_Tmpldef_HeadFile(buf, t, c);
			buf += u8"\n";
		}
	}
	buf += u8"}\n";

	buf += u8"#undef _HeadFileScope\n";
	buf += u8"#endif\n";

	return buf;
}

std::u8string CppBuilder::buildCppFile(elpModule* t) {
	std::u8string buf;
	buf += u8"#include " + ResolveDotExprToFile(t->getDotExpr(), u8"h") + u8"\n";
	buf += u8"using namespace " + ResolveDotExprToNS(t->getDotExpr(), true) + u8";\n\n";
	for (auto c : *t->queryAllClasses()) {
		if (!c->getClassMark()->assemblied_class) {
			continue;
		}
		if (c->getClassMark()->subscript_class) {
			continue;
		}
		for (auto a : c->queryAttributes()) {
			if (a->getMark().static_attribute) {
				buf += ResolveClassDefToNotion(a->attribute_class, qualify_policy(t, a->attribute_class->visitModule()));
				buf += u8" ";
				std::vector<std::u32string> dotExpr = { c->getName(), a->name };
				buf += ResolveDotExprToNS(dotExpr, false) + u8";\n";
			}
		}
		for (auto f : c->queryFunctions()) {
			auto mark = f->getMark();
			if (mark.null_function) {
				continue;
			}
			if (f->visitExecutable() == nullptr) {
				continue;
			}
			buf += ResolveClassDefToNotion(f->visitReturnClass(), qualify_policy(t, f->visitReturnClass()->visitModule()));
			buf += u8" ";
			{
				std::vector<std::u32string> dotExpr = { c->getName(), f->getName() };
				buf += ResolveDotExprToNS(dotExpr, false);
			}
			buf += u8"(";
			Build_ParamterPrototype(buf, t, *f->visitParamterClasses(), *f->visitParamterNames());
			buf += u8")";
			if (!mark.mutable_function) { buf += u8" const"; }
			buf += u8" ";
			Build_Function_Block(buf, t, c, f, u8"  ");
			buf += u8"\n";
		}
	}
	return buf;
}

#endif