#pragma once

#include <string>
#include <vector>

#include "Components/Parser/TokenKind.h"

class LexicalRule {
public:

	class TokenTableEntry {
	public:
		TokenKind kind;
		std::vector<std::u32string>& patterns;
	};

	class OperationPrecedenceEntry {
	public:
		std::u32string random;
		int precedence;
	};

public:
	std::vector<std::u32string> Breakings = {
		U"++", U"--", U"&&", U"||",
		U"<=", U">=", U"==", U"<", U">",
		U".", U"->",
		U"+", U"-", U"*", U"/",
		U"&", U"|",
		U"!",
		U"=",
		U";",
		U"::",
		U":",
		U",",
	};

	std::vector<std::u32string> Brackets = {
		U"(", U")", U"[", U"]", U"{", U"}"
	};

	std::vector<std::u32string> Keywords = {
		U"包",
		U"导入",
		U"导出",
		U"可以之类",
		U"守拙之类",
		U"举",
		U"伪",
		U"推",
		U"伺",
		U"避",
		U"退",
		U"返回",
		U"表现",
		U"类属性",
		U"常量",
		U"#t",
		U"#f",
		U"设立",
		U"静态存储",
		U"边界继承",
		U"演绎"
	};

	std::vector<TokenTableEntry> BreakingSet = {
		{ TokenKind::random,    Breakings },
		{ TokenKind::bracket,   Brackets }
	};

	std::vector<TokenTableEntry> KeywordSet = {
		{ TokenKind::keyword,        Keywords },
	};

	std::vector<std::u32string> UnaryOperations = {
		U"+",
		U"-",
		U"++",
		U"--",
		U"!",
		U"*"
	};

	bool isIdentifierLetter(char32_t c) {
		return (((c >= '0') && (c <= '9'))
			|| ((c >= 'a') && (c <= 'z'))
			|| ((c >= 'A') && (c <= 'Z'))
			|| (c == '_')
			|| (c > 127));
	}

	std::vector<OperationPrecedenceEntry> OperationPrecedenceTable = {
		{ U"=", 1 },

		{ U"<=", 5 },
		{ U">=", 5 },
		{ U"<",  5 },
		{ U">",  5 },
		{ U"==", 5 },
		{ U"!=", 5 },

		{ U"+", 11 },
		{ U"-", 11 },

		{ U"*", 12 },
		{ U"/", 12 },
		{ U"&", 12 },
		{ U"|", 12 },
		{ U"&&", 12 },
		{ U"||", 12 },
	};

	std::vector<std::u32string> UnaryOperationTable = {
		U"::",
		U"+",
		U"-",
		U"++",
		U"--",
		U"!",
		U"*",
		U"&",
	};

};


extern LexicalRule SLexicalRule;

class DefaultConfig {
public:
	 const std::u8string srcFileExt = u8".shui";
};

extern DefaultConfig SDefaultConfig;