#include "Inspector.h"

#include "unicode.h"

#include <iostream>

void Inspector::print(ExprComponent* expr) {
	if (expr->kind == ExprComponentKind::unary) {
		std::cout << "(unary " << To_CppString(expr->op) << " ";
		print(expr->S[0]);
		std::cout << ")";
	}
	else if (expr->kind == ExprComponentKind::binary) {
		std::cout << "(binary " << To_CppString(expr->op) << " ";
		print(expr->S[0]);
		std::cout << " ";
		print(expr->S[1]);
		std::cout << ")";
	}
	else if (expr->kind == ExprComponentKind::quantity) {
		std::cout << "(quantity " << To_CppString(expr->quantity) << ")";
	}
	else if (expr->kind == ExprComponentKind::identifier) {
		std::cout << "(identifier " << To_CppString(expr->identifier) << ")";
	}
	else if (expr->kind == ExprComponentKind::bracket) {
		std::cout << "#" << To_CppString(expr->op);
		for (int i = 0; i < expr->sp->size(); ++i) {
			if (i > 0) { std::cout << " "; }
			print((*expr->sp)[i]);
		}
		std::cout << (char) pairBracket(expr->op[0]);
		std::cout << ")";
	}
	else if (expr->kind == ExprComponentKind::chunk) {
		std::cout << "(chunk";
		for (int i = 0; i < expr->sp->size(); ++i) {
			std::cout << " ";
			print((*expr->sp)[i]);
		}
		std::cout << ")";
	}
	else {
		std::cout << "ERROR";
	}
}

/*
empty_statement,
expr_statement,
if_statement,
for_statement,
for_iter_statement,
while_statement,
break_statement,
continue_statement,
return_statement,
block_statement,
*/

void Inspector::print(StatementComponent* statement) {
	if (statement->kind == StatementComponentKind::empty_statement) {
		std::cout << "(空语句)";
	}
	else if (statement->kind == StatementComponentKind::expr_statement) {
		std::cout << "(表现式 ";
		print(statement->SC[0]);
		std::cout << ")";
	}
	else if (statement->kind == StatementComponentKind::if_statement) {
		std::cout << "(举 ";
		print(statement->SC[0]);
		std::cout << " ";
		print(statement->S[0]);
		if (statement->S[1] != nullptr) {
			std::cout << " ";
			print(statement->S[1]);
		}
		std::cout << ")";
	}
	else if (statement->kind == StatementComponentKind::for_statement) {
		std::cout << "(推 (";
		print(statement->SC[0]);
		std::cout << ";";
		print(statement->SC[1]);
		std::cout << ";";
		print(statement->SC[2]);
		std::cout << ") (";
		print(statement->S[0]);
		std::cout << ")";
		std::cout << ")";
	}
	else if (statement->kind == StatementComponentKind::for_iter_statement) {
		std::cout << "(推 (";
		print(statement->SC[0]);
		std::cout << " : ";
		print(statement->SC[1]);
		std::cout << ") (";
		print(statement->S[0]);
		std::cout << ")";
		std::cout << ")";
	}
	else if (statement->kind == StatementComponentKind::while_statement) {
		std::cout << "(伺 (";
		print(statement->S[0]);
		std::cout << ")";
		std::cout << ")";
	}
	else if (statement->kind == StatementComponentKind::continue_statement) {
		std::cout << "(避)";
	}
	else if (statement->kind == StatementComponentKind::break_statement) {
		std::cout << "(退)";
	}
	else if (statement->kind == StatementComponentKind::return_statement) {
		std::cout << "(返回";
		if (statement->SC[0]) {
			std::cout << " ";
			print(statement->SC[0]);
		}
		std::cout << ")";
	}
	else if (statement->kind == StatementComponentKind::block_statement) {
		std::cout << "(块";
		for (auto s : *statement->sp) {
			std::cout << " ";
			print(s);
		}
		std::cout << ")";
	}
	else {
		std::cout << "ERROR";
	}
}

void Inspector::print(FunctionDefinitionComponent* func_def) {
	std::cout << "(func";
	if (func_def->ml->is_export) { std::cout << " export"; }
	if (func_def->ml->is_edge) { std::cout << " edge"; }
	if (func_def->ml->is_static) { std::cout << " static"; }
	if (func_def->ml->mutable_mark) { std::cout << " mutable"; }
	if (func_def->ml->perform_mark) { std::cout << " perform"; }
	std::cout << " " << To_CppString(func_def->ml->name);
	std::cout << " (return-type ";
	print(func_def->ml->rt); 
	std::cout << ") " << "(args ";
	print(func_def->ml->sp);
	std::cout << ") ";
	if (func_def->ml->S == nullptr) {
		std::cout << "(未定义)";
	}
	else {
		print(func_def->ml->S);
	}
	std::cout << ")";
}

void Inspector::print(ClassDefinitionComponent* class_def) {
	std::cout << "(class";
	if (class_def->is_export) { std::cout << " 导出"; }
	if (class_def->is_second) { std::cout << " 守拙之类"; }
	if (class_def->is_template) { std::cout << " 模板类"; }
	std::cout << " " << To_CppString(class_def->name);
	for (auto s : *class_def->attrs) {
		if (s->is_function) {
			std::cout << " ";
			print(s->func);
		}
		else {
			std::cout << " (attr " << To_CppString(s->name) << ")";
		}
	}
	std::cout << ")";
}

#if 0
	ExprComponent* rt = nullptr;
	std::u32string name;
	bool is_export = false;
	bool is_edge = false;
	bool is_static = false;
	bool is_perform = false;
	bool mutable_mark = false;
	bool perform_mark = false;
	ExprComponent* sp = nullptr;
	StatementComponent* S = nullptr;
#endif