#pragma once

#include "Components/ErrorBuffer/ErrorBuffer.h"
#include "Components/CompileProcess/CompileProcess.h"
#include "Components/ModuleComponent/ExprComponent.h"
#include "Components/Parser/Token.h"

class NameBase;
class ModuleComponent;

class ModuleValidation
{
public:
	inline void SetCompileProcess(CompileProcess& cp) {
		compileProcess = &cp;
	}
	inline void SetErrorBuffer(ErrorBuffer* errorBuffer) {
		this->errorBuffer = errorBuffer;
	}
	inline void pushError(const std::u8string& fileName, const Token& token, const std::u8string& s) {
		if (errorBuffer) {
			errorBuffer->push(fileName, token.line_pos, token.column_pos, s);
		}
		else {
			SErrorBuffer.push(fileName, token.line_pos, token.column_pos, s);
		}
	}
public:
	bool CheckClassPremiseEq(ClassDefinitionComponent* a, ClassDefinitionComponent* b);
	bool LoadModuleSymbol(NameBase& nb, ModuleComponent& Module);
public:
	bool checkSpecifiedClassNotionSyntax(const std::u8string& file_name, ExprComponent* expr);
	bool checkFunctionParamsListSyntax(const std::u8string& file_name, ExprComponent* expr);
	bool checkTypeCaptureSyntax(const std::u8string& file_name, ExprComponent* expr);
	bool checkExpressionClassSyntax(const std::u8string& file_name, std::vector<ExprComponent*>& expr);
	static bool checkImportModuleNotNesting(ModuleComponent* new_import, ModuleComponent* outer);
public:
	bool ValidateClassDefinition(NameBase& nb, ModuleComponent* mod, ClassDefinitionComponent* cdef);
	ModuleComponent* ValidateModule(ModuleComponent& NotValidatedModule);
private:
	ErrorBuffer* errorBuffer = nullptr;
	CompileProcess* compileProcess = nullptr;
};