#include "NameBase.h"

void NameBase::putScope(ScopeKind kind, ModuleComponent* m, ClassDefinitionComponent* c, FunctionDefinitionComponent* f) {
	auto sc = new Scope;
	sc->kind = kind;
	if (kind == ScopeKind::module_def) {
		sc->m = m;
	}
	else if (kind == ScopeKind::class_def) {
		sc->c = c;
	}
	else if (kind == ScopeKind::function) {
		sc->f = f;
	}
	scopes.push_back(sc);
}

void NameBase::popScope() {
	if (scopes.size()) {
		auto sc = scopes.back();
		scopes.pop_back();
		delete sc;
	}
}

Scope* NameBase::getScope(ScopeKind kind) {
	for (auto iter = scopes.rbegin(); iter != scopes.rend(); ++iter) {
		if ((*iter)->kind == kind) {
			return *iter;
		}
	}
	return nullptr;
}

void NameBase::putEntry(std::u32string name, SpecifiedClassNotion* v) {
	if (scopes.size() == 0) {
		putScope(ScopeKind::block, nullptr, nullptr, nullptr);
	}
	auto se = new ScopeEntry(ScopeEntryKind::variable);
	se->name = name;
	se->type = v;
	scopes.back()->es.push_back(se);
}

void NameBase::putEntry(std::u32string name, FunctionDefinitionComponent* f) {
	if (scopes.size() == 0) {
		putScope(ScopeKind::block, nullptr, nullptr, nullptr);
	}
	auto se = new ScopeEntry(ScopeEntryKind::function_entry);
	se->name = name;
	se->f = f;
	scopes.back()->es.push_back(se);
}

void NameBase::putEntry(std::u32string name, ClassDefinitionComponent* cls) {
	if (scopes.size() == 0) {
		putScope(ScopeKind::block, nullptr, nullptr, nullptr);
	}
	auto se = new ScopeEntry(ScopeEntryKind::class_entry);
	se->name = name;
	se->c = cls;
	scopes.back()->es.push_back(se);
}

ScopeEntry* NameBase::getEntry(std::u32string name) {
	for (auto iter = scopes.rbegin(); iter != scopes.rend(); ++iter) {
		for (auto entry : (*iter)->es) {
			if (entry->name == name) {
				return entry;
			}
		}
	}
	return nullptr;
}