#pragma once

#include <string>
#include <vector>

class FunctionDefinitionComponent;
class ClassDefinitionComponent;
class ModuleComponent;

class SpecifiedClassNotion;

enum class ScopeKind
{
	block,
	for_block,
	while_block,
	function,
	class_def,
	module_def,
	reverse
};

enum class ScopeEntryKind
{
	class_entry,
	function_entry,
	variable,
	reverse
};

class ScopeEntry
{
public:
	inline ScopeEntry(ScopeEntryKind kind_): kind(kind_){}
	std::u32string name;
	ScopeEntryKind kind;
	FunctionDefinitionComponent* f = nullptr;
	ClassDefinitionComponent* c = nullptr;
	SpecifiedClassNotion* type = nullptr;
};

class Scope
{
public:
	ScopeKind kind;
	FunctionDefinitionComponent* f = nullptr;
	ClassDefinitionComponent* c = nullptr;
	ModuleComponent* m = nullptr;
	std::vector<ScopeEntry*> es;
};

class NameBase
{
public:
	std::vector<Scope*> scopes;
	void putScope(ScopeKind kind, ModuleComponent* m, ClassDefinitionComponent* c, FunctionDefinitionComponent* f);
	void popScope();
	Scope* getScope(ScopeKind kind);
	void putEntry(std::u32string name, SpecifiedClassNotion* cls);
	void putEntry(std::u32string name, FunctionDefinitionComponent* cls);
	void putEntry(std::u32string name, ClassDefinitionComponent* cls);
	ScopeEntry* getEntry(std::u32string name);
};