#include "Scanner.h"

void Scanner::initScanner(const std::u8string& file_path, const std::u32string& chunk) {
	this->chunk = chunk;
	this->file_name = file_path;
	line_pos = 1;
	column_pos = 1;
	indent = 0;
	indentEnd = false;
	next = 0;
	end = chunk.size() - 1;
}

const std::u32string& Scanner::getChunkData() {
	return chunk;
};

void Scanner::dropOrEnd() {

	//
	//
	//
	if (next > end) {
		return;
	}

	char32_t c = chunk[next];
	if (c == '\r') {
		++line_pos;
		column_pos = 1;
		indent = 0;
		indentEnd = false;

		//
		//
		//
		++next;
		if (chunk[next] == '\n') {

			//
			//
			//
			++next;
		}
		return;
	}
	if (c == '\n') {
		++line_pos;
		column_pos = 1;
		indent = 0;
		indentEnd = false;

		//
		//
		//
		++next;
		return;
	}
	if ((c == ' ') || (c == '\t') && !indentEnd) {
		++indent;
	}
	else {
		indentEnd = true;
	}
	++column_pos;

	//
	//
	//
	++next;
	return;
}

char32_t Scanner::copyOrEnd() {
	if (next <= end) {
		return chunk[next];
	}
	else {
		return 0;
	}
}

char32_t Scanner::dropAndCopy() {
	dropOrEnd();
	return copyOrEnd();
}

bool Scanner::testSequence(const std::u32string& s) {
	if (next + s.size() > end) {
		return false;
	}
	if (s.size() == 2) {
		return (chunk[next] == s[0])
			&& (chunk[next + 1] == s[1]);
	}
	int sp = next;
	for (int i = 0; i < s.size(); ++i) {
		if (chunk[sp] != s[i]) {
			return false;
		}
		++sp;
	}
	return true;
}

void Scanner::skipBlank() {
	while (next <= end) {
		char32_t c = copyOrEnd();
		if ((c == ' ') || (c == '\t') || (c == '\n')) {
			dropOrEnd();
		}
		else {
			break;
		}
	}
}

int Scanner::skipLineOrEnd() {
	int c = 0;
	while (next <= end) {
		if (copyOrEnd() != '\n') {
			dropOrEnd();
			++c;
			continue;
		}
		else {
			dropOrEnd();
			++c;
			return c;
		}
	}
	return c;
}

int Scanner::skipUntilStringOrEnd(const std::u32string& s) {
	int c = 0;
	while (next <= end) {
		if (testSequence(s)) {
			return c;
		}
		dropOrEnd();
		++c;
	}
	return c;
}

std::u32string Scanner::copyUntilStringAndDrop(const std::u32string& s) {
	size_t sp = next;
	while (next <= end) {
		if (testSequence(s)) {
			size_t ep = next;
			for (int i = 0; i < s.size(); ++i) {
				dropOrEnd();
			}
			return chunk.substr(sp, ep - sp);
		}
		dropOrEnd();
	}
	return U"";
}

int Scanner::skipCountOrEnd(int n) {
	int c = 0;
	for (int i = 0; i < n; ++i)
	{
		if (next <= end) {
			dropOrEnd();
			++c;
		}
		else {
			break;
		}
	}
	return c;
}

const std::u8string& Scanner::getFileName() {
	return file_name;
}

int Scanner::getLine() {
	return line_pos;
}

int Scanner::getColumn() {
	return column_pos;
}

int Scanner::getIndent() {
	return indent;
}

bool Scanner::isEnd() {
	return copyOrEnd() == 0;
}

void Scanner::setEnd() {
	next = end + 1;
}