
#include "unicode.h"
#include "Token.h"

Token::Token() {
	kind = TokenKind::reverse;
	value = std::u32string();
	line_pos = column_pos = -1;
}

Token::Token(TokenKind kind, std::u32string value, int line_pos, int column_pos) {
	this->kind = kind;
	this->value = value;
	this->line_pos = line_pos;
	this->column_pos = column_pos;
}

bool Token::testRandom(const std::u32string& s) {
	return this && (kind == TokenKind::random)
		&& (value == s);
}

bool Token::isIdentifier() {
	return this && kind == TokenKind::identifier;
}

bool Token::isKeyword(const std::u32string& s) {
	return this && (kind == TokenKind::keyword)
		&& (value == s);
}

bool Token::isOpenBracket() {
	return this && (kind == TokenKind::bracket)
		&& ((value == U"(") || (value == U"[") || (value == U"{"));
}

bool Token::isCloseBracket() {
	return this && (kind == TokenKind::bracket)
		&& ((value == U")") || (value == U"]") || (value == U"}"));
}

bool Token::isBracket(const std::u32string& s) {
	return this && (kind == TokenKind::bracket)
		&& (value == s);
}

std::string Token::makePrintable() {
	return "(" + std::to_string(line_pos) + ", " + std::to_string(column_pos) + ") " + To_CppString(value);
}
