#pragma once

#include <optional>

#include "Scanner.h"
#include "Token.h"

class ErrorBuffer;

class Tokenizer
{
public:
	~Tokenizer();
	Scanner& queryScanner();
	Tokenizer();
	Tokenizer(const Tokenizer& other) = delete;
	Tokenizer& operator=(const Tokenizer& other) = delete;
	void initTokenizer(const std::u8string& file_path, const std::u32string& chunk, ErrorBuffer* error_buffer);
	void dropOrEnd();
	std::optional<Token> copyOrEnd();
	std::optional<Token> dropAndCopy();
	bool isEnd();
	void setEnd();

public:
	static Tokenizer* createTokenizer(const std::u8string& file_path, ErrorBuffer* error_buffer);

public:
	void pushError(int line_pos, int column_pos, const std::u8string& error_string);

private:
	Scanner* scanner;
	std::optional<Token> token;
	ErrorBuffer* error_buffer;
};