#include "PathConfig.h"

#include "unicode.h"

#include "Components/DefaultConfig/DefaultConfig.h"

#include <filesystem>
#include <fstream>

bool PathConfig::AddDir(const std::u8string& file_path) {
	SearchDirs.push_back(file_path);
	return true;
}

std::u8string PathConfig::ResolvePackageRootPath(const std::vector<std::u32string> pt, const std::u8string& file_path) {
	std::filesystem::path fp = file_path;
	for (auto s = pt.rbegin(); s != pt.rend(); ++s) {
		if (fp.parent_path().filename().u32string() != *s) {
			return nullptr;
		}
		fp = fp.parent_path();
	}
	fp = fp.parent_path();
	return fp.u8string();
}

std::u8string GetSubPath(const std::u8string& root_path, const std::vector<std::u32string> pt) {
	std::filesystem::path fp = root_path;
	for (auto s : pt) {
		fp /= s;
	}
	return fp.u8string();
}

std::u8string PathConfig::searchPackage(const std::vector<std::u32string>& pt) {
	for (auto s = SearchDirs.rbegin(); s != SearchDirs.rend(); ++s) {
		std::u8string file_path = GetSubPath(*s, pt);
		if (file_path.size()) {
			if (SDefaultConfig.srcFileExt.size()) {
				file_path += SDefaultConfig.srcFileExt;
			}
			std::filesystem::path fp = file_path;
			if (std::filesystem::exists(fp)) {
				return fp.u8string();
			}
		}
	}
	return {};
}
std::u32string PathConfig::loadFileContent(const std::u8string& file_path) {
	std::string file_path_cppstr = To_CppString(file_path);
	std::ifstream ifs(To_CppString(file_path));
	std::string content_cpp((std::istreambuf_iterator<char>(ifs)),
		(std::istreambuf_iterator<char>()));
	std::u8string content = reinterpret_cast<const char8_t*>(content_cpp.c_str());
	return To_UTF32(content);
}

bool PathConfig::checkPackage(const std::vector<std::u32string>& pt, const std::u8string& file_path) {
	std::filesystem::path fp = file_path;
	for (auto s = pt.rbegin(); s != pt.rend(); ++s) {
		if (fp.parent_path().filename().u32string() != *s) {
			return false;
		}
		fp = fp.parent_path();
	}
	fp = fp.parent_path();
	return true;
}