#pragma once

#include <string>

inline std::u8string To_UTF8(const std::u32string& s)
{
	std::u8string utf8;
	for (char32_t c : s)
	{
		auto i = (uint32_t)c;
		if (i < 0x80)
		{
			utf8.push_back((char)i);
		}
		else if (i < 0x800)
		{
			utf8.push_back((char)(0xc0 | (i >> 6)));
			utf8.push_back((char)(0x80 | (i & 0x3f)));
		}
		else if (i < 0x10000)
		{
			utf8.push_back((char)(0xe0 | (i >> 12)));
			utf8.push_back((char)(0x80 | ((i >> 6) & 0x3f)));
			utf8.push_back((char)(0x80 | (i & 0x3f)));
		}
		else if (i < 0x200000)
		{
			utf8.push_back((char)(0xf0 | (i >> 18)));
			utf8.push_back((char)(0x80 | ((i >> 12) & 0x3f)));
			utf8.push_back((char)(0x80 | ((i >> 6) & 0x3f)));
			utf8.push_back((char)(0x80 | (i & 0x3f)));
		}
		else
		{
			utf8.push_back((char)(0xf8 | (i >> 24)));
			utf8.push_back((char)(0x80 | ((i >> 18) & 0x3f)));
			utf8.push_back((char)(0x80 | ((i >> 12) & 0x3f)));
			utf8.push_back((char)(0x80 | ((i >> 6) & 0x3f)));
			utf8.push_back((char)(0x80 | (i & 0x3f)));
		}
	}
	return utf8;
}

inline std::u32string To_UTF32(const std::u8string& s)
{
	if (s.empty())
	{
		return {};
	}

	std::u32string res;

	for (size_t i = 0; i < s.size(); )
	{
		auto c = (unsigned char)s[i];
		char32_t wideChar = 0;
		if ((c & 0x80) == 0)
		{
			wideChar = c;
			++i;
		}
		else if ((c & 0xE0) == 0xC0)  ///< 110x-xxxx 10xx-xxxx
		{
			if (i + 2 > s.size()) break;
			wideChar = (char32_t(c) & 0x3F) << 6;
			wideChar |= (char32_t(s[i + 1]) & 0x3F);
			i += 2;
		}
		else if ((c & 0xF0) == 0xE0)  ///< 1110-xxxx 10xx-xxxx 10xx-xxxx
		{
			if (i + 3 > s.size()) break;
			wideChar = (char32_t(c) & 0x1F) << 12;
			wideChar |= (char32_t(s[i + 1]) & 0x3F) << 6;
			wideChar |= (char32_t(s[i + 2]) & 0x3F);
			i += 3;
		}
		else if ((c & 0xF8) == 0xF0)  ///< 1111-0xxx 10xx-xxxx 10xx-xxxx 10xx-xxxx
		{
			if (i + 4 > s.size()) break;
			wideChar = (char32_t(c) & 0x0F) << 18;
			wideChar |= (char32_t(s[i + 1]) & 0x3F) << 12;
			wideChar |= (char32_t(s[i + 2]) & 0x3F) << 6;
			wideChar |= (char32_t(s[i + 3]) & 0x3F);
			i += 4;
		}
		else///< 1111-10xx 10xx-xxxx 10xx-xxxx 10xx-xxxx 10xx-xxxx
		{
			if (i + 4 > s.size()) break;
			wideChar = (char32_t(c) & 0x07) << 24;
			wideChar |= (char32_t(s[i + 1]) & 0x3F) << 18;
			wideChar |= (char32_t(s[i + 2]) & 0x3F) << 12;
			wideChar |= (char32_t(s[i + 3]) & 0x3F) << 6;
			wideChar |= (char32_t(s[i + 4]) & 0x3F);
			i += 4;
		}
		res.push_back(wideChar);
	}
	return res;
}

inline std::u8string To_UTF8(const std::string& s) {
	return reinterpret_cast<const char8_t*>(s.c_str());
}

inline std::string To_CppString(const std::u8string& s) {
	return reinterpret_cast<const char*>(s.c_str());
}

inline std::string To_CppString(const std::u32string& s) {
	return To_CppString(To_UTF8(s));
}